/*=========================================================================
 * CallNauty_packed.c - mex file which calls Nauty using the packed graph 
 * format.
 * Needs to be linked with the following files:
 *
 * nauty.c
 * nautil.c
 * naugraph.c
 * nautinv.c
 *
 * Program Description:
 * Returns automorphism group generators (S), stabilized vertices (vstab),
 * group size (Gsize), orbits (orb), canonical labeling (lab), and 
 * canonical adjacency matrix (Acanon) for the graph represented by an 
 * adjacency matrix input A with coloring matrix c. Coloring is optional. 
 * The input matrix A may be type double or the more memory efficient type 
 * int8. The output matrix Acanon is returned as the same data type as the 
 * input matrix A. S, vstab, Gsize, orb, lab, and c are always type double.
 *
 * Inputs:
 *      A: n by n binary matrix where n is the number of vertices of a 
 *         graph.
 *      c: 2 by n matrix. The first row lists the vertices (vertices are
 *         labeled 1 to n) in some order such that vertices of the same 
 *         color are continuous. The second row splits the listed vertices
 *         into cells corresponding to different color classes. The ends of
 *         the color classes are indicated by zeros.
 *
 * Outputs:
 *      S: m by n matrix of permutations of the set {1,...,n} corresponding
 *         to generating permutations of the automorphism group of A.
 *      vstab: 1 by m vector listing the stabilized vertex associated with 
 *             each generator in S (See Nauty documentation, section 6, for
 *             a more detailed explanation). The unique elements in vstab 
 *             form a base for the group generated by S and, along with S, 
 *             may be used to identify Schreier generators for the 
 *             point-wise stabilizers of elements in the base. 
 *      Gsize: number representing the size of the group generated by S.
 *      orb: 1 by n array containing the orbits of the automorphism 
 *         group. orb(i) is the number of the least numbered vertex in
 *         the same orbit as i.
 *      lab: 1 by n array listing the vertices in the order they need to be
 *         relabelled to give Acanon.
 *      Acanon: n by n binary matrix corresponding to the adjacency matrix
 *         of the canonically labelled isomorph of the graph represented by
 *         A.
 * 
 * The calling syntax is:
 *
 *		[S,vstab,Gsize,orb,lab,Acanon] = CallNauty_packed(A,c)
 *
 * See also, CallNauty_sparse.c, CallNauty_quick.c
 *             
 * Author:   Scott Bugenhagen
 * Contact:  sbugenha@mcw.edu
 * Date:     8/30/2011
 *
 * References:
 * 1. Brendan D. McKay. Nauty user's guide (version 2.4). Australian 
 *    National University, 2009. retrieved from 
 *    <http://cs.anu.edu.au/~bdm/nauty/> on 6/1/2011.
 *=======================================================================*/

#include <math.h>
#include "nauty.h"
#include "nautinv.h"
#include "mex.h"

int *generators; /* global pointer to generators */
int *stabverts; /* global pointer to stabilized vertices */

/*=======================================================================*/
/* The gateway routine */

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
  graph *g, *canong;
  int *lab, *ptn, *orbits;
  static DEFAULTOPTIONS_GRAPH(options);
  statsblk stats;
  setword *workspace;
  set *grow;
  int m,n,mrows,ncols,i,j,numgens;
  double Gsize;
  double *output[4];
  void GetGenerators(int, permutation *, int *, int, int, int);
  int setcmp(void *, int);
  
  generators = NULL;
  stabverts = NULL;

  /* note: all the options fields below are fixed in this program with the
   * exception of the defaultptn field. If a coloring matrix is supplied as
   * input, defaultptn is set to FALSE. If no coloring is given, it remains
   * as TRUE. Beware that the canonical labeling may differ depending on 
   * the setting of this option. Note that though digraph is always set to 
   * TRUE, the program can still handle simple graphs (though performance 
   * may be slightly worse than if the setting were FALSE). The invariant
   * "adjacencies" is applied as recommended in the following link
   * http://dcsmail.anu.edu.au/pipermail/nauty-list/2010-May/000556.html
   * for digraphs. */
  
  options.cartesian = TRUE;
  options.digraph = TRUE;
  options.getcanon = TRUE;
  options.userautomproc = GetGenerators;
  options.defaultptn = TRUE;
  options.invarproc = adjacencies;
  options.mininvarlevel = 0;
  options.maxinvarlevel = 99;
  
  /* check input/output for proper number of arguments */
  if(nrhs > 2) 
    mexErrMsgTxt("Too many input arguments.");
  if(nrhs == 0)
    mexErrMsgTxt("Too few input arguments.");
  if(nlhs > 6) 
    mexErrMsgTxt("Too many output arguments.");
  
  /* check format of adjacency matrix */
  if( !mxIsNumeric(prhs[0]) || mxIsComplex(prhs[0]) )
    mexErrMsgTxt("Adjacency matrix must be noncomplex numeric.");
  mrows = mxGetM(prhs[0]);
  ncols = mxGetN(prhs[0]);
  n = ncols;
  if (mrows != ncols)
      mexErrMsgTxt("Adjacency matrix must be square.");
  
  /* check format of coloring matrix if given */
  if (nrhs == 2)
  {
      if( !mxIsNumeric(prhs[1]) || mxIsComplex(prhs[1]) )
        mexErrMsgTxt("Coloring matrix must be noncomplex numeric.");

      mrows = mxGetM(prhs[1]);
      ncols = mxGetN(prhs[1]);
      if( mrows != 2 )
        mexErrMsgTxt("Coloring matrix must contain 2 rows.");
      if( ncols != n )
       mexErrMsgTxt("Coloring and adjacency matrix column lengths must be equal.");
      
      options.defaultptn = FALSE;
  }
  
  /* allocate space for intermediate data */
  m = (int)ceil((double)n/(double)WORDSIZE);
  g = (graph *) mxMalloc(n*m*sizeof(graph));
  if (g == NULL)
      mexErrMsgTxt("mxMalloc failed.");
  lab = (int *) mxMalloc(n*sizeof(int));
  if (lab == NULL)
      mexErrMsgTxt("mxMalloc failed.");
  ptn = (int *) mxMalloc(n*sizeof(int));
  if (ptn == NULL)
      mexErrMsgTxt("mxMalloc failed.");
  orbits = (int *) mxMalloc(n*sizeof(int));
  if (orbits == NULL)
      mexErrMsgTxt("mxMalloc failed.");
  workspace = (setword *) mxMalloc(5*m*sizeof(setword));
  if (workspace == NULL)
      mexErrMsgTxt("mxMalloc failed.");
  canong = (graph *) mxMalloc(n*m*sizeof(graph));
  if (canong == NULL)
      mexErrMsgTxt("mxMalloc failed.");

  /* unpack inputs to format required by Nauty */  
  switch(mxGetClassID(prhs[0]))
  {
      case mxDOUBLE_CLASS : {
          double *input;
          input = (double *) mxGetData(prhs[0]);
          for (i = 0; i < n; i++) {
              grow = GRAPHROW(g,i,m);
              EMPTYSET(grow,m);
              for (j = 0; j < n; j++) {
                  if (input[j*n + i] != 0.0)
                      ADDELEMENT(grow,j); }}}
          break;
                                
      case mxINT8_CLASS : {
          char *input;
          input = (char *) mxGetData(prhs[0]);
          for (i = 0; i < n; i++) {
              grow = GRAPHROW(g,i,m);
              EMPTYSET(grow,m);
              for (j = 0; j < n; j++) {
                  if (input[j*n + i] != 0)
                      ADDELEMENT(grow,j); }}}
          break;
          
      default :  
          mexErrMsgTxt("Adjacency matrix is not of supported class type.");                          
  }
  
  if (nrhs == 2)
  {
      switch(mxGetClassID(prhs[1]))
      {
          case mxDOUBLE_CLASS : {
              double *input;
              input = (double *) mxGetData(prhs[1]);
              for (j = 0; j < n; j++) {
                      lab[j] = (int) input[j*2] - 1;
                      ptn[j] = (int) input[j*2 + 1]; }}
              break;
              
          /* Currently, only type double is supported for the coloring 
           * matrix. If other types are required, additional cases may
           * be added here. */    

          default :
           mexErrMsgTxt("Coloring matrix must be of type double.");                          
      }
      if (!setcmp((void *)lab, n))
          mexErrMsgTxt("Illegal coloring matrix. See help.");
  }
  
  nauty_check(WORDSIZE,m,n,NAUTYVERSIONID); /* check if linking to 
                                             * compatible versions of
                                             * Nauty routines */
  /* call the Nauty routine */
  /* note: This program does not allow active cells to be set and uses
   * the default (all cells active). Therefore, active is set to NULL. */

  nauty(g,lab,ptn,NULL,orbits,&options,&stats,
										   workspace,5*m,m,n,canong);
  numgens = stats.numgenerators;
  Gsize = stats.grpsize1*pow(10.0,(double)stats.grpsize2);

  /* pack output into mxArrays */
  /* note: mxArrays corresponding to generators, orbits, and labeling data 
   * always return type double matrices in this version. If other data 
   * types are required, output may be cast to the required type in the 
   * MATLAB command window using MATLAB internal functions. */
  
  plhs[0] = mxCreateDoubleMatrix(numgens, n, mxREAL); /* S */
  plhs[1] = mxCreateDoubleMatrix(1, numgens, mxREAL); /* vstab */
  plhs[2] = mxCreateDoubleScalar(Gsize);              /* Gsize */
  plhs[3] = mxCreateDoubleMatrix(1, n, mxREAL);       /* orbits */
  plhs[4] = mxCreateDoubleMatrix(1, n, mxREAL);       /* lab */
  
  output[0] = mxGetPr(plhs[0]);
  output[1] = mxGetPr(plhs[1]);
  for (i = 0; i < numgens; i++) {
      output[1][i] = (double) stabverts[i] + 1.0;
      for (j = 0; j < n; j++) {
          output[0][j*numgens + i] = (double) generators[i*n + j] + 1.0; }}
  
  output[2] = mxGetPr(plhs[3]);
  for (i = 0; i < n; i++) {
      output[2][i] = (double) orbits[i] + 1.0; }
  
  output[3] = mxGetPr(plhs[4]);
  for (i = 0; i < n; i++) {
      output[3][i] = (double) lab[i] + 1.0; }
  
  /* note: mxArray corresponding to canonically labeld graph returns an
   * adjacency matrix of the same data type as the input. */
  
  switch(mxGetClassID(prhs[0]))
  {
      case mxDOUBLE_CLASS : {
          double *output4;
          plhs[5] = mxCreateNumericMatrix(n,n,mxDOUBLE_CLASS,mxREAL);
          output4 = mxGetData(plhs[5]);
          for (i = 0; i < n; i++) {
              grow = GRAPHROW(canong,i,m);
              for (j = 0; j < n; j++) {
                  if ( ISELEMENT(grow,j) )
                      output4[j*n + i] = 1.0; }}}
          break;
          
      case mxINT8_CLASS : {
          char *output4;
          plhs[5] = mxCreateNumericMatrix(n,n,mxINT8_CLASS,mxREAL);
          output4 = mxGetData(plhs[5]);
          for (i = 0; i < n; i++) {
              grow = GRAPHROW(canong,i,m);
              for (j = 0; j < n; j++) {
                  if ( ISELEMENT(grow,j) )
                      output4[j*n + i] = 1; }}}
          break;                          
  }
  
  /* clean-up */
  mxFree(g);
  mxFree(lab);
  mxFree(ptn);
  mxFree(orbits);
  mxFree(workspace);
  mxFree(canong);
  mxFree(generators);
  mxFree(stabverts);
}

/*=======================================================================*/
/* User defined procedures */

void GetGenerators(int count, permutation *perm, int *orbits, 
                                  int numorbits, int stabvertex, int n)
{
    int i;
    int *tmpgens;
    int *tmpstabs;
    
    tmpgens = (int *)mxRealloc(generators, count * n * sizeof(int));
    tmpstabs = (int *)mxRealloc(stabverts, count*sizeof(int));
    
    if (tmpgens != NULL && tmpstabs != NULL) {
        generators = tmpgens;
        stabverts = tmpstabs; }
    else {
        mxFree(generators);
        mxFree(stabverts);
        mexErrMsgTxt("mxRealloc failed."); }

    for (i = 0; i < n; i++) {
        generators[(count - 1) * n + i] = (int)perm[i]; }
    stabverts[(count - 1)] = stabvertex;
}

int setcmp(void *set, int num)
{
    void *sortset;
    int i, flag;
    int cmpels(const void *, const void *);
    
    sortset = mxMalloc(num*sizeof(int));
    if (sortset == NULL)
      mexErrMsgTxt("mxMalloc failed.");
    memcpy(sortset, set, num*sizeof(int));
    
    qsort(sortset, num, sizeof(int), cmpels);
    
    flag = 1;
    for (i = 0; i < num; i++) {
        if ( ((int *)sortset)[i] != i ) {
            flag = 0; 
            break; }}
    
    mxFree(sortset);
    return(flag);
}

int cmpels(const void *arg1, const void *arg2)
{
    return( *(int *)arg1 - *(int *)arg2 );
}