function [fcode] = Encode(f,Tsize,Trelab,numdigits,startofdigit,endofdigit)
%ENCODE encodes a function f representing a number in an arbitrary number
%system to numdigits double precision decimal numbers.
%
% INPUT:  f is a row vector or an array of row vectors representing finite-
%         valued functions which are system states.
%
%         Tsize is a row vector representing a map from each state s in S 
%         to a component size, which is the number of states connected to
%         s (i.e. the cardinality of the subset Si containing s). The
%         coding scheme regards Tsize(f(i)) for each i as the base of the 
%         ith digit of f with f representing a number in an arbitrary 
%         number system.
%
%         Trelab is a row vector representing a function which relabels 
%         each state s belonging to Si such that the set of labels are 
%         {0,1,...,Tsize(s)-1}.
%
%         numdigits is the integer number of double precision digits
%         (decimal numbers) required to encode a function f.
%
%         startofdigit is a 1 by numdigits row vector which is an index 
%         into the first (most significant) digit of f belonging to an 
%         encoded decimal digit such that startdigit(i) is the first digit 
%         of f belonging to the ith decimal digit.
%
%         endofdigit is a 1 by numdigits row vector which is an index into 
%         the last (least significant) digit of f belonging to an encoded 
%         decimal digit such that startdigit(i) is the last digit of f 
%         belonging to the ith decimal digit.
%
% OUTPUT: fcode is a m by numdigits array, where ith rows is the encoded
%         form of the ith function in f.
%
% See also, Decode, GenerateCodingScheme, IDComponentTypes
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011

fcode = zeros(size(f,1),numdigits);
for i = 1:numdigits
    bprod = 1;
    fcode(:,i) = Trelab(f(:,endofdigit(i)))';
    for j = endofdigit(i)-1:-1:startofdigit(i)
        bprod = bprod*Tsize(f(1,j+1));
        digval = Trelab(f(:,j))*bprod;
        fcode(:,i) = fcode(:,i) + digval(:);
    end
end
end