function [fcanon] = FindCanonRep(f,d,v,e,c,base,T)
%FINDCANONREP finds the canonical representative of a system state f.
%
% DESCRIPTION:
% A function K is a system state canonizer if, for all f,f' in [f], where 
% [f] is the equivalence class of f under the action of a group G, 
% K(f) = K(f') = fcanon, where fcanon is the canonical representative
% system state in [f]. FindCanonRep is a system state canonizer which is
% based on Theorem 4.5 in ref. 1.
%
% INPUT:  f is a numeric vectors which is a function of D with
%         values in S, representing a system state.
%
%         d,v,e, and c represent the system characteristic graph, where:
%         d is a 1 by n array such that d(i) represents the out-degree of 
%         each vertex i = 1,2,...,n;
%
%         v is a 1 by n array such that v(i) is an index into array e such
%         that e(v(i)),e(v(i)+1),...,e(v(i)+d(i)-1) are the vertices to
%         which vertex i is connected;
%
%         e is a 1 by q array, with q being the number of edges, listing
%         the sets of vertices to which each vertex i is connected as
%         described above;
%
%         and c is a 2 by n matrix. The first row lists the vertices in 
%         some order such that vertices of the same color are continous. 
%         The second row splits the listed vertices into different color
%         classes. The ends of the color classes are indicated by zeros.
%
%         base and T are a Schreier-Sims representation of the group G,
%         where:
%         base is a numeric vector representing a base for the group G;
%
%         and T is a length(base) by 1 cell array representing a strong
%         generating set for G. Each cell i contains the coset
%         representatives of the subgroup of G stabilizing base(1:i) in the
%         subgroup of G stabilizing base(1:i-1).
%
% OUTPUT: fcanon is a numeric vector representing the canonical
%         representative of system state f.
%
% Notes: This function calls CallNauty_sparse.c and FindCompatPerms.
%
% See also, SchreierSims, SetupNautyGraph_sparse
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References: 
% 1. Junttila T. On the symmetry reduction method for petri nets and 
%    similar formalisms (doctoral thesis). Helsinki, Finland: Helsinki 
%    University of Technology, 2003.

f = f(:);
p = length(f);
[fsort,ixf] = sort(f);
[~,ends] = unique(fsort);
c(1,1:p) = ixf;
c(2,1:p) = ones(1,p);
c(2,ends) = 0;
[~,~,~,~,gammainv] = CallNauty_sparse(d,v,e,c);
gamma(gammainv) = 1:length(gammainv);
g = FindCompatPerms_injective(gamma(1:p),base,T);
fcanon = f(g(1:p))';
end