function [pi] = FindIndependentSubsets(G,finit,T,rules,D,rho)
%FINDINDEPENDENTSUBSETS computes the connectivity of the system
%characteristic graph.
%
% INPUT:  G is a q by 4 cell array representing the characteristic
%         graph of the system, where q is the number of hyperarcs in the 
%         graph. Each row j in G represents a hyperarc, with column 1 
%         containing the tail sets, column 2 the head sets, column 3 the 
%         component state coloring functions, and column 4 the rate 
%         constant colors.
%
%         finit is a row vector representing a finite-valued function which 
%         is the root state in the system reachability graph.
%
%         T is a row vector representing the partition {S1,S2,...,Sr} of 
%         the set S of component states such that the value of T(s) is the 
%         same for all s in Si and different for all s not in Si. 
%
%         rules is a 4 column numeric array representing the transition 
%         rules. The first column contains indices into the set Q of 
%         component transitions, the second column contains indices into 
%         the set C of rate constants, the third column contains indices 
%         into the set D of dependency matrices, and the fourth column 
%         contains a flag indicating the format of its associated 
%         dependency matrix (zero indicates spatial coupling format, a 
%         nonzero indicates mean field coupling format). Each row j is a 
%         mapping from the component transition Q(rules(j,1)) to the rate 
%         constant C(rules(j,2)) under the restriction that the system
%         state satisfies dependency matrix D(rules(j,3)). If a rate 
%         constant cj in C is not associated with a dependency graph (i.e. 
%         has no functional dependencies), then rules(j,3) should be set to 
%         zero (rules(j,4) is ignored in this case).
%
%         D is a cell vector of dependency matrices. A dependency matrix in
%         spatial coupling format is given as a q by 3 cell array (where q 
%         is the number of hyperarcs in the dependency graph) with the 
%         first column containing the tail sets Ay, the second containing 
%         the head sets By, and the third containing the coloring function 
%         Ry of each yth hyperarc such that rho{Ry(j)} is the color of 
%         By(j). A dependency matrix in mean field coupling format is given
%         as a 2 row matrix with the first row containing indices into rho,
%         and the second containing an integer value such that D{i}(2,j) is
%         the number of components required to be in the state class 
%         D{i}(1,j).
%
%         rho is cell vector of component state classes. Each cell contains
%         a unique subset of the set S of component states.
%
% OUTPUT: pi is a partition of the vertices of G into maximally connected
%         subsets such that the value pi(i) is the same for each ith vertex
%         belonging to a given maximally connected subset and different for
%         each vertex not belonging the subset.
%
% See also, MergeSubsets, FindUniqueSubsystems
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References: 
% 1. Bugenhagen, SM et al. Manuscript in preparation.

p = length(finit);
q = size(G,1);
pi = zeros(1,p);
unprocessed = 1:p;
queue = zeros(1,p-1);
while any(unprocessed)
    source = unprocessed(find(unprocessed,1,'first'));
    pi(source) = source;
    queue(1) = source;
    while any(queue)
        v = queue(find(queue,1,'first'));
        queue(queue==v) = 0;
        unprocessed(unprocessed==v) = 0;
        for y = 1:q
            AUB = union(G{y,1},G{y,2});
            if ismember(v,AUB)
                d = setdiff(intersect(AUB,unprocessed),queue);
                if ~isempty(d)
                    pi(d) = source;
                    ixq = find(~queue,length(d),'first');
                    queue(ixq) = d;
                end
            end
        end
    end
end
% The code below was added to accomodate mean-field coupling which is not
% accounted for in the characteristic graph
for Dix = rules(rules(:,3) & rules(:,4),3)'
    Dprime = D{Dix};
    b = unique(T(rho{Dprime(1,1)}));
    for x = 2:size(Dprime,2)
        b = union(b,T(rho{Dprime(1,x)}));
    end
    ixpi = ismember(T(finit),b);
    b = unique(pi(ixpi));
    for x = 2:length(b)
        d = pi == b(x);
        pi(d) = b(1);
    end
end         
end