function [T,Tsize,Trelab] = IDComponentTypes(n,Q)
%IDCOMPONENTTYPES generates functions that can be used to characterize
%components.
%
% INPUT:  n is an integer representing the size of the set S of all 
%         component states.
%         
%         Q is a cell array representing the set of component transitions
%         such that Q{j,1} represents the source states Xj, Q{j,2}
%         represents the mapping Yj from Xj to the destinations states in 
%         S, and Q{j,3} represents the multiplicity function Zj. 
%         Specifically, a transition j involves Q{j,3}(i) components in 
%         state Q{j,1}(i), each of which map to state Q{j,2}(i), for 
%         i = 1,...,length(Q{j,1}).
%
% OUTPUT: T is a row vector representing the partition {S1,S2,...,Sr} of 
%         the set S of component states such that the value of T(s) is the 
%         same for all s in Si and different for all s not in Si. Si
%         contains the set of reachable states for a component of type i.
%
%         Tsize is a row vector representing a map from each state s in S 
%         to a component size, which is the number of states connected to
%         s (i.e. the cardinality of the subset Si containing s).
%
%         Trelab is a row vector representing a function which relabels 
%         each state s belonging to Si such that the set of labels are 
%         {0,1,...,Tsize(s)-1}.
%
% See also, FindIndependentSubsets, MergeSubsets, FindUniqueSubsets
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References: 
% 1. Bugenhagen, SM et al. Manuscript in preparation.

T = zeros(1,n);
m = size(Q,1);
unprocessed = 1:n;
queue = zeros(1,n);
while any(unprocessed)
    source = unprocessed(find(unprocessed,1,'first'));
    T(source) = source;
    queue(1) = source;
    while any(queue)
        s = queue(find(queue,1,'first'));
        queue(queue==s) = 0;
        unprocessed(unprocessed==s) = 0;
        for j = 1:m
            if ismember(s,union(Q{j,1},Q{j,2}))
                if ismember(s,Q{j,1})
                    Ys = Q{j,2}(find(Q{j,1}==s,1));
                    if ismember(Ys,setdiff(unprocessed,queue))
                        T(Ys) = source;
                        ixq = find(~queue,1,'first');
                        queue(ixq) = Ys;
                    end
                end
                if ismember(s,Q{j,2})
                    Yinvs = Q{j,1}(find(Q{j,2}==s,1));
                    if ismember(Yinvs,setdiff(unprocessed,queue))
                        T(Yinvs) = source;
                        ixq = find(~queue,1,'first');
                        queue(ixq) = Yinvs;
                    end
                end
            end
        end
    end
end
[Tsort,S] = sort(T);
[~,uniquelast] = unique(Tsort,'last');
Tsortsize(1:uniquelast(1)) = uniquelast(1);
Tsortrelab(1:uniquelast(1)) = 0:uniquelast(1)-1;
for i=2:length(uniquelast)
    Tsortsize(uniquelast(i-1)+1:uniquelast(i)) = uniquelast(i) - ...
                                                           uniquelast(i-1);
    Tsortrelab(uniquelast(i-1)+1:uniquelast(i)) = 0:(uniquelast(i) - ...
                                                      uniquelast(i-1) - 1); 
end
Tsize(S) = Tsortsize;
Trelab(S) = Tsortrelab;
end