function [base,orb,v,T] = SchreierSims(S,vstab)
%SCHREIERSIMS generates a Schreier-Sims representation of the permutation
%group generated by S.
%
% DESCRIPTION:
% Assume a finite set X of size n and permutation group G on that set. A
% base for G is a sequence of points in X such that the stabilizer of all
% the points is the identity. Given base (x1,...,xd), let G0 = G, 
% G(i) = stab(xi,G(i-1)), and G(d) = {1} for i = 1,...,d.. For i = 1,...,d, 
% calculate a set T(i) of coset representatives for G(i) in G(i-1). The 
% union of all T(i) is a strong generating set for G. The base and strong 
% generating set together are a Schreier-Sims representation of the group 
% G.
%
% INPUT:  S is an m by n numeric array representing a generating set of m
%         permutations of a set of size n.
%
%         vstab is an m-length numeric vector listing the stabilized points 
%         associated with each generator in S. The unique elements in vstab 
%         form a base for the group generated by S and, along with S, are 
%         used to identify Schreier generators for subgroups G(0) to 
%         G(d-1). vstab is an output of the CallNauty functions (See Nauty 
%         documentation, section 6, for a more detailed explanation).
%
% OUTPUT: base is d-length numeric vector representing a base for the group
%         generated by S.
%
%         orb is a d by 1 cell array with each cell containing a numeric
%         vector representing the orbit of point base(i) under subgroup
%         G(i-1).
%
%         v is a d by 1 cell array with each cell containing a Schreier
%         vector for orb{i}. A Schreier vector is a list of labels of each
%         jth point orb{i}(j) with the index of the permutation in S 
%         carrying a previous point in orb{i} to the point orb{i}(j). The
%         point orb{i} == base(i) is the start point, and is labeled as -1.
%         It is a compact representation of the strong generating set T
%         (see below).
%
%         T is a d by 1 cell array with each cell containing a
%         length(orb{i}) by n array representing the coset representatives
%         for G(i-1) in G. T is a strong generating set for G.
%
% See also, CallNauty_packed.c, CallNauty_sparse.c
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References: 
% 1. Peter J. Cameron. Combinatorics: topics, techniques, algorithms. pp. 
%    234-237. Cambridge University Press, 1994.

n = size(S,2);
base = unique(vstab(end:-1:1));
d = length(base);
orb = cell(d,1);
v = cell(d,1);
for i = 1:d
    Y = zeros(1,n);
    lab = zeros(1,n);
    unprocessed = zeros(1,n-1);
    Y(1) = base(i);
    lab(1) = -1;
    Yixend = 1;
    unprocessed(1) = 1;
    unprocessedixend = 1;
    Six = 1:find(vstab == base(i),1,'last');
    while any(unprocessed)
        x = Y(unprocessed(unprocessedixend));
        unprocessed(unprocessedixend) = 0;
        unprocessedixend = unprocessedixend - 1;
        for j = Six
            xprime = S(j,x);
            if ~ismember(xprime,Y)
                Yixend = Yixend + 1;
                Y(Yixend) = xprime;
                lab(Yixend) = j;
                unprocessedixend = unprocessedixend + 1;
                unprocessed(unprocessedixend) = Yixend;
            end
        end
    end
    orb{i} = Y(1:Yixend);
    v{i} = lab(1:Yixend);
end
if nargout == 4
    Sinv = zeros(size(S));
    for i = 1:size(S,1)
        Sinv(i,S(i,:)) = 1:n;
    end
    T = cell(d,1);
    for i = 1:d
        lab = v{i};
        x = base(i);
        Y = orb{i};
        s = length(lab);
        k = zeros(s,n);
        k(1,:) = 1:n;
        for j = 2:s
            g = Sinv(lab(j),:);
            y = g(Y(j));
            while y ~= x
                g = Sinv(lab(Y == y),g);
                y = g(Y(j));
            end
            k(j,g) = 1:n;
        end
        T{i} = k;
    end
end
end