function [d,v,e,c] = SetupNautyGraph_sparse(Gtilde)
%SETUPNAUTYGRAPH_SPARSE converts a characteristic graph Gtilde from the 
%format produced by the GenerateCharacteristicGraph function to a sparse 
%format readable by Nauty.
%
% INPUT:  Gtilde is a vertex-colored equivalent of a characteristic graph G
%         which is an output of the function GeenerateCharacteristicGraph. 
%         (See GenerateCharacteristicGraph help for more details).
%
% OUTPUT: d is a 1 by n array such that d(i) represents the out-degree of 
%         each vertex i = 1,2,...,n.
%
%         v is a 1 by n array such that v(i) is an index into array e such
%         that e(v(i)),e(v(i)+1),...,e(v(i)+d(i)-1) are the vertices to
%         which vertex i is connected.
%
%         e is a 1 by q array, with q being the number of edges, listing
%         the sets of vertices to which each vertex i is connected as
%         described above.
%
%         c is a 2 by n matrix. The first row lists the vertices in some
%         order such that vertices of the same color are continous. The
%         second row splits the listed vertices into different color
%         classes. The ends of the color classes are indicated by zeros.
%
% See also, SetupNautyGraph_packed, CallNauty_sparse.c
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References:
% 1. McKay, BD. Nauty user's guide (version 2.4). Australian National 
%    University, 2009. Retrieved from <http://cs.anu.edu.au/~bdm/nauty/> 
%    on 6/1/2011.

L = Gtilde{1,1};
Utilde = Gtilde{1,2};
nv = length(L);
ne = size(Utilde,1);
c = ones(2,nv);
[Lsort,c(1,:)] = sort(L);
[~,ends] = unique(Lsort);
c(2,ends) = 0;
if ~isempty(Utilde)
    tails = Utilde(:,1);
    heads = Utilde(:,2);
else
    tails = [];
    heads = [];
end
[tailssort,eix] = sort(tails);
[vix,ends] = unique(tailssort,'last');
[~,starts] = unique(tailssort,'first');
v = ones(1,nv);
d = zeros(1,nv);
e = zeros(1,ne);
v(vix) = starts;
d(vix) = ends - starts + 1;
if ne == 0
    e = 1;
else
    e(1,:) = heads(eix);
end
end